# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

from driverng import Worklist

parser_class = "cobas"

QUERY = """
H|\\^&|||cobas6000^1|||||host|TSREQ^REAL|P|1
Q|1|^^                   123456^0^5134^3^^S1^SC^R2||ALL||||||||O
L|1|N
"""


def test_query(self):
    result = self.decode(QUERY)
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "123456"}],
    }
    assert result == expected


def test_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].rack = "foo-bar-zaz-luz"
    result = self.order(worklist=worklist)
    expected = """
        H|\\^&|||MCA||||||TSDWN^BATCH|1|1|20130715141912
        P|1|||L036666666|^NYME^ANO||20110513|F||||||1^Y
        O|1|123456|0^foo^bar^^zaz^luz|^^^A^1:10\\^^^B|R||20130513114242||||A||||Serum||||||||||O
        C|1|I|NYME ANO NJF^PRESCRIBER^SER|G
        O|2|123456|0^foo^bar^^zaz^luz|^^^C|R||20130513114242||||C||||Serum||||||||||O
        C|1|I|NYME ANO NJF^PRESCRIBER^SER|G
        L|1|N
        """
    assert result == expected


def test_encoder_order_without_prescriber(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0]["physicians"] = []
    result = self.order(worklist=worklist)
    expected = """
        H|\\^&|||MCA||||||TSDWN^BATCH|1|1|20130715141912
        P|1|||L036666666|^NYME^ANO||20110513|F||||||1^Y
        O|1|123456||^^^A^1:10\\^^^B|R||20130513114242||||A||||Serum||||||||||O
        C|1|I|NYME ANO NJF^^SER|G
        O|2|123456||^^^C|R||20130513114242||||C||||Serum||||||||||O
        C|1|I|NYME ANO NJF^^SER|G
        L|1|N
        """
    assert result == expected


def test_query_order(self):
    result = self.order(context={"query": self.parse(QUERY)})
    expected = """
        H|\\^&|||MCA||||||TSDWN^REPLY|1|1|20130715141912
        P|1|||L036666666|^NYME^ANO||20110513|F||||||1^Y
        O|1|123456|0^5134^3^^S1^SC|^^^A^1:10\\^^^B|R||20130513114242||||A||||Serum||||||||||O
        C|1|I|NYME ANO NJF^PRESCRIBER^SER|G
        O|2|123456|0^5134^3^^S1^SC|^^^C|R||20130513114242||||C||||Serum||||||||||O
        C|1|I|NYME ANO NJF^PRESCRIBER^SER|G
        L|1|N
        """
    assert result == expected


def test_voidOrder(self):
    result = self.order(
        worklist=Worklist(context="order"), context={"query": self.parse(QUERY)}
    )
    expected = """
        H|\\^&|||MCA||||||TSDWN^REPLY|1|1|20130715141912
        L|1|I
        """
    assert result == expected


def test_result(self):
    result = self.decode(
        """
    H|\\^&|||cobas6000^1|||||host|RSUPL^REAL|P|1
    P|1|||||||F||||||83^Y
    O|1|              98765432100|0^5134^3^^S1^SC|^^^210\\^^^675\\^^^992\\^^^993\\^^^994|R||||||N||||1|||||||20140109151343|||F
    C|1|I|TEST Essai NOMJF           ^^^^ |G
    R|1|^^^210/|0.4|mg/L||H||F||gb    |20140109163825||P1
    C|1|I|43|I
    M|1|ECR|BMSERV|^^^1|RackPack|E12|198781|710|0|196420|O|I^O^--------------------\\M^O^-----\\S^O^--------------------\\R^1.00\\D^O^-----\\F^O\\Y^O^-----|1242^1185^0.000^0.000\\24581^24542^0.000^0.000|0.000^1.51^0.000^0.000^0.000|uIU/mL|||20000529175246
    R|2|^^^675/|39^POS|mg/L||N||F||gb    |20140109164001||P1
    C|1|I|0|I
    R|3|^^^992/|6|||N||F||gb    |20140109163943||P1
    C|1|I|0|I
    R|4|^^^993/|9|||N||F||gb    |20140109163943||P1
    C|1|I|0|I
    R|5|^^^994/|1|||N||F||gb    |20140109163943||P1
    C|1|I|0|I
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "98765432100",
                "rack": "5134-3-S1-SC",
                "results": [
                    {
                        "flags": ["H", "43"],
                        "instrument": "P1",
                        "date": datetime.datetime(2014, 1, 9, 16, 38, 25),
                        "lots": [
                            {"name": "198781", "type": "reagent", "lot": "98765432100"},
                            {
                                "name": "196420",
                                "type": "calibration",
                                "lot": "98765432100",
                            },
                        ],
                        "value": "0.4",
                        "code": "210",
                        "operator": "gb",
                    },
                    {
                        "flags": ["N"],
                        "instrument": "P1",
                        "date": datetime.datetime(2014, 1, 9, 16, 40, 1),
                        "value": "39",
                        "code": "675",
                        "operator": "gb",
                        "lots": [],
                    },
                    {
                        "flags": ["N"],
                        "instrument": "P1",
                        "date": datetime.datetime(2014, 1, 9, 16, 40, 1),
                        "value": "POS",
                        "code": "675Q",
                        "operator": "gb",
                        "lots": [],
                    },
                    {
                        "flags": ["N"],
                        "instrument": "P1",
                        "date": datetime.datetime(2014, 1, 9, 16, 39, 43),
                        "value": "6",
                        "code": "992",
                        "operator": "gb",
                        "lots": [],
                    },
                    {
                        "flags": ["N"],
                        "instrument": "P1",
                        "date": datetime.datetime(2014, 1, 9, 16, 39, 43),
                        "value": "9",
                        "code": "993",
                        "operator": "gb",
                        "lots": [],
                    },
                    {
                        "flags": ["N"],
                        "instrument": "P1",
                        "date": datetime.datetime(2014, 1, 9, 16, 39, 43),
                        "value": "1",
                        "code": "994",
                        "operator": "gb",
                        "lots": [],
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_qc(self):
    result = self.decode(
        """
    H|\\^&|||cobas6000^1|||||host|RSUPL^REAL|P|1
    P|1|||||||U||||||^
    O|1|17139900              |24002^3018^4^^QC^SC|^^^20\\^^^116\\^^^148\\^^^195\\^^^216|||||||Q||||1|||||||20131112100733|||F
    C|1|I|^^^^ |G
    R|1|^^^20//not|52.26|mIU/mL||N||||jv    |20131112103126||E11
    R|2|^^^116//not|307.0|IU/mL||N||||jv    |20131112103208||E11
    C|1|I|0|I
    R|3|^^^148//not|43.03|mIU/mL||N||||jv    |20131112103250||E11
    C|1|I|0|I
    R|4|^^^195//not|25.33|pmol/L||N||||jv    |20131112103332||E11
    C|1|I|0|I
    R|5|^^^216//not|2.32^foo|ng/mL||N||||jv    |20131112103414||E11
    C|1|I|0|I
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "17139900",
                "control_lot": "17139900",
                "results": [
                    {
                        "instrument": "E11",
                        "date": datetime.datetime(2013, 11, 12, 10, 31, 26),
                        "lots": [],
                        "value": "52.26",
                        "code": "20",
                        "operator": "jv",
                        "flags": ["N"],
                    },
                    {
                        "instrument": "E11",
                        "date": datetime.datetime(2013, 11, 12, 10, 32, 8),
                        "lots": [],
                        "value": "307.0",
                        "code": "116",
                        "operator": "jv",
                        "flags": ["N"],
                    },
                    {
                        "instrument": "E11",
                        "date": datetime.datetime(2013, 11, 12, 10, 32, 50),
                        "lots": [],
                        "value": "43.03",
                        "code": "148",
                        "operator": "jv",
                        "flags": ["N"],
                    },
                    {
                        "instrument": "E11",
                        "date": datetime.datetime(2013, 11, 12, 10, 33, 32),
                        "lots": [],
                        "value": "25.33",
                        "code": "195",
                        "operator": "jv",
                        "flags": ["N"],
                    },
                    {
                        "instrument": "E11",
                        "date": datetime.datetime(2013, 11, 12, 10, 34, 14),
                        "lots": [],
                        "value": "2.32",
                        "code": "216",
                        "operator": "jv",
                        "flags": ["N"],
                    },
                    {
                        "instrument": "E11",
                        "date": datetime.datetime(2013, 11, 12, 10, 34, 14),
                        "lots": [],
                        "value": "foo",
                        "code": "216Q",
                        "operator": "jv",
                        "flags": ["N"],
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_erreur(self):
    date = datetime.datetime(2014, 4, 11, 12, 20, 18)
    result = self.decode(
        """
    H|\\^&|||cobas6000^1|||||host|RSUPL^REAL|P|1
    P|1|||||||M||||||58^Y
    O|1|              23983800|0^5058^2^^S1^SC|^^^990^\\^^^989^\\^^^991^|R||20140411074700||||N||||1|||||||20140411112537|||F
    R|1|^^^990/|      |mmol/L||A||F||gb    |20140411122018||ISE1
    C|1|I|72|I
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "23983800",
                "rack": "5058-2-S1-SC",
                "results": [
                    {
                        "flags": ["A", "72"],
                        "lots": [],
                        "instrument": "ISE1",
                        "date": date,
                        "value": "ERREUR",
                        "code": "990",
                        "operator": "gb",
                    }
                ],
            }
        ],
    }
    assert result == expected
